package com.dsic.gj_erp.aspect;


import com.dsic.gj_erp.annotation.AuthFunction;
import com.dsic.gj_erp.exception.CustomException;
import com.dsic.gj_erp.service.ApiService;
import com.dsic.gj_erp.util.JwtUtil;
import io.jsonwebtoken.Claims;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Pointcut;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import javax.servlet.http.HttpServletRequest;

@Aspect
@Component
public class AuthFunctionAspect {

    @Autowired
    private HttpServletRequest request;
    @Autowired
    private JwtUtil jwtUtil;
    @Autowired
    private ApiService apiService;


    /**
     * Controller层切点 注解拦截
     */
    @Pointcut("@annotation(authFunction) ")
    public void doAuthFunction(AuthFunction authFunction) {
    }


    /**
     * 功能权限认证
     */
    @Around("doAuthFunction(authFunction)")
    public Object deBefore(ProceedingJoinPoint pjp, AuthFunction authFunction) throws Throwable {
        String token = request.getHeader("token");
        if (token == null) {
            throw new CustomException(10105, jwtUtil.getHeader() + "不能为空！");
        }
        Claims claims = jwtUtil.getClaimByToken(token);
        if (claims == null || jwtUtil.isTokenExpired(claims.getExpiration())) {
            throw new CustomException(10106, jwtUtil.getHeader() + "失效，请重新登录！");
        }
        request.setAttribute("yhms",claims.get("yhms"));
        request.setAttribute("yhdm", claims.getId());
        request.setAttribute("zxtdm", claims.getSubject());
        if (!apiService.hasPermission(claims.getId(), authFunction.CKDMS())) {
            throw new CustomException(10104, "用户没有该菜单权限！");
        }
        return pjp.proceed();
    }

}
