package com.dsic.gj_erp.controller.kban;

import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.dsic.gj_erp.bean.ResultBean;
import com.dsic.gj_erp.bean.jcsj.DmBzry;
import com.dsic.gj_erp.bean.jhgk.DmYdjh;
import com.dsic.gj_erp.exception.ServiceException;
import com.dsic.gj_erp.pc.Constant;
import com.dsic.gj_erp.service.KbanService;
import com.dsic.gj_erp.service.jcsj.DmBzryService;
import com.dsic.gj_erp.service.zyjh.ZyjhService;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.*;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Collectors;


@RestController
@RequestMapping("kban")
@AllArgsConstructor
public class KbanController {

    private final ZyjhService zyjhService;
    private final DmBzryService cqService;
    private final KbanService kbanService;

    /**
     * 获取所有未处理批次信息
     */
    @GetMapping("wcl")
    public ResultBean<?> getWcl(){
        return new ResultBean<>(zyjhService.getWcl());
    }

    /**
     * 看板线表计划相关数据
     */
    @GetMapping("xbjh")
    public ResultBean<?> kban(String nf){
        if (ObjUtil.isEmpty(nf)){
            throw new ServiceException(10001,"年份不能为空");
        }

        List<Double> xbjh=kbanService.getEveryMonXbzlByNf(nf);
        List<Map<String, Object>> comp = kbanService.getEveryCompZlByNf(nf);

        HashMap<Object, Object> map = new HashMap<>();
        map.put("xbjh", xbjh);
        map.put("comp", comp);
        return new ResultBean<>(map);
    }

    /**
     * 计划跟踪
     */
    @GetMapping("getJhgz")
    public ResultBean<?> getJhgz(){
        return new ResultBean<>(kbanService.getJhgz());
    }

    /**
     * 按照设备编号查询作业计划
     */
    @GetMapping("getZyjhDetailBySbCode")
    public ResultBean<?> getZyjhDetailBySbCode(String code){
        if (ObjUtil.isEmpty(code)){
            throw new ServiceException(10001,"设备编号不能为空");
        }
        return new ResultBean<>(kbanService.getZyjhDetailBySbCode(code));
    }

    /**
     * 统计划线、切割设备本人计划及作业成功
     */
    @GetMapping("getSb")
    public ResultBean<?> getSb(){
        List<Map<String, Object>> sb = kbanService.getSb();
        return new ResultBean<>(sb);
    }

    /**
     * 按照人员工号查询当月所有参与的任务
     */
    @GetMapping("getZjjhByUserCode")
    public ResultBean<?> getZjjhByUserCode(String userCode){
        if (ObjUtil.isEmpty(userCode)){
            throw new ServiceException(10001,"用户不能为空");
        }
        return new ResultBean<>(kbanService.getZjjhByUserCode(userCode));
    }

    /**
     * 按照跨位查询作业计划明细
     */
    @GetMapping("getKwDetailByKw")
    public ResultBean<?> getEveryKw(String kw){
        if (ObjUtil.isEmpty(kw)){
            throw new ServiceException(10001,"垮位不能为空");
        }

        QueryWrapper<DmYdjh> wrapper = new QueryWrapper<>();
        if ("1".equals(kw)){
            wrapper.lambda().ge(DmYdjh::getZt, 11);
        }

        if (Arrays.asList("2","3","4").contains(kw)){
            wrapper.eq("SUBSTRING(tzbh,6,1)", kw);
            wrapper.ge("zt", "42");
        }
        return new ResultBean<>(zyjhService.list(wrapper));
    }

    /**
     * 本月各跨位情况
     */
    @GetMapping("getEveryKw")
    public ResultBean<?> getEveryKw(){
        Map<String, Object> map = kbanService.getEveryKw();
        return new ResultBean<>(map);
    }

    /**
     * 在建船只,各船只进度
     */
    @GetMapping("gcjd")
    public ResultBean<?> gcjd(){
        return new ResultBean<>(zyjhService.gcjd());
    }

    /**
     * 各垮计划完成情况
     */
    @GetMapping("gxjhwcqk")
    public ResultBean<?> gxjhwcqk(){
        List<Map<String, Object>> gxjhwcqk = zyjhService.gxjhwcqk();
        return new ResultBean<>(gxjhwcqk);
    }

    /**
     * 各月份产量(吨)
     */
    @GetMapping("gyfcl")
    public ResultBean<?> gyfcl(){
        return new ResultBean<>(zyjhService.gyfcl());
    }

    /**
     * 计划完成情况(吨)
     */
    @GetMapping("jhwcqk")
    public ResultBean<?> jhwcqk(){
        return new ResultBean<>(zyjhService.jhwcqk());
    }

    /**
     * 各垮产量
     */
    @GetMapping("gekuachaliang")
    public ResultBean<?> gekuachaliang(){
        return new ResultBean<>(zyjhService.gekuachaliang());
    }

    /**
     * 考勤情况
     */
    @GetMapping("kq")
    public ResultBean<?> kq(){
        List<DmBzry> ryList=cqService.list(Wrappers.<DmBzry>lambdaQuery()
                .isNotNull(DmBzry::getUserCode).isNotNull(DmBzry::getZt).ne(DmBzry::getZt,""));

        List<DmBzry> collect = ryList.stream().filter(item -> "01".equals(item.getZt())).collect(Collectors.toList());

        Map<String,Object> map=new HashMap<>();
        map.put("total",ryList.size());
        map.put("rows",ryList);
        map.put("bb", collect.size());
        map.put("eb",ryList.size()-collect.size());
        return new ResultBean<>(map);
    }
}
