package com.dsic.gj_erp.controller.zyjh;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.dsic.gj_erp.annotation.AuthFunction;
import com.dsic.gj_erp.bean.ResultBean;
import com.dsic.gj_erp.bean.jhgk.DmYdjh;
import com.dsic.gj_erp.bean.zyjh.Ycldw;
import com.dsic.gj_erp.bean.zyjh.YcldwInfo;
import com.dsic.gj_erp.controller.zyjh.dto.*;
import com.dsic.gj_erp.exception.ServiceException;
import com.dsic.gj_erp.service.zyjh.YcldwInfoService;
import com.dsic.gj_erp.service.zyjh.YcldwService;
import com.dsic.gj_erp.service.zyjh.ZyjhService;
import com.dsic.gj_erp.ws.WsHandler;
import com.dsic.gj_erp.wsclient.Message;
import com.dsic.gj_erp.wsclient.WebSocketService;
import lombok.AllArgsConstructor;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.lang.reflect.Field;
import java.util.List;

@RestController
@RequestMapping("zyjh")
@AllArgsConstructor
public class ZyjhController {

    private final ZyjhService zyjhService;

    private final YcldwService ycldwService;

    private final YcldwInfoService ycldwInfoService;

    private final WsHandler wsHandler;

    private final WebSocketService webSocketService;

    /**
     * 作业计划执行,派工&反馈
     */
    @PostMapping("zx")
    public ResultBean<?> zx(@RequestBody PgDto dto) {
        if (StrUtil.isEmpty(dto.getZxZtName())) {
            throw new ServiceException(10001,"作业计划执行内容不能为空");
        }
        List<DmYdjh> list=dto.getList();
        try {
            Field field=DmYdjh.class.getDeclaredField(dto.getField());
            if (ObjUtil.isNotEmpty(field)){
                field.setAccessible(true);
                String date=DateUtil.date().toString("yyyy/MM/dd HH:mm:ss");
                for(DmYdjh dmYdjh:list){
                    field.set(dmYdjh,date);
                    dmYdjh.setJhlx(null);
                    dmYdjh.setZtByZyjhzx(dto.getZxZtName());
                }
            }
            if ("PWFK".equals(dto.getZxZtName())){
                ycldwInfoService.updatePw(dto.getYcldwInfo());
                wsHandler.sendPwComplete((JSONObject) JSONObject.toJSON(dto.getYcldwInfo()));
            }
            if ("HXFK".equals(dto.getZxZtName())){
                ycldwInfoService.updatePw(dto.getYcldwInfo());
                wsHandler.sendHXComplete((JSONObject) JSONObject.toJSON(dto.getYcldwInfo()));
            }
            if ("QGFK".equals(dto.getZxZtName())){
                //无需划线,从理料间获取时才更新垛位信息
                if ("42".equals(dto.getZt())){
                    ycldwInfoService.updatePw(dto.getYcldwInfo());
                }
                wsHandler.sendHXComplete((JSONObject) JSONObject.toJSON(dto.getYcldwInfo()));
                wsHandler.sendQGComplete((JSONObject) JSONObject.toJSON(dto.getYcldwInfo()));
            }
            zyjhService.updateBatchById(list);

            if (dto.getAudioFlag()){
                Message result = Message.result(webSocketService.getErpId(), "1");
                webSocketService.sendMessage(result.toString());
            }

            return new ResultBean<>();
        } catch (NoSuchFieldException | IllegalAccessException e) {
            e.printStackTrace();
            if (dto.getAudioFlag()){
                Message result = Message.result(webSocketService.getErpId(), "0");
                try {
                    webSocketService.sendMessage(result.toString());
                } catch (Exception ex) {
                    e.printStackTrace();
                    throw new ServiceException(10001,"执行失败");
                }
            }
            throw new ServiceException(10001,"反馈属性不存在");
        } catch (Exception e) {
            e.printStackTrace();
            throw new ServiceException(10001,"执行失败");
        }
    }
    @PostMapping("zxweb")
    @AuthFunction
    public ResultBean<?> zxweb(@RequestBody PgDto dto,HttpServletRequest request) {
        if (StrUtil.isEmpty(dto.getZxZtName())) {
            throw new ServiceException(10001,"作业计划执行内容不能为空");
        }
        List<DmYdjh> list=dto.getList();

        try {
            Field field=DmYdjh.class.getDeclaredField(dto.getField());
            Field fieldry=DmYdjh.class.getDeclaredField(dto.getFieldry());
            if (ObjUtil.isNotEmpty(field)){
                field.setAccessible(true);
                String date=DateUtil.date().toString("yyyy/MM/dd");
                for(DmYdjh dmYdjh:list){
                    field.set(dmYdjh,date);
                    dmYdjh.setZtByZyjhzx(dto.getZxZtName());

                    if(ObjUtil.isNotEmpty(fieldry)){
                        fieldry.setAccessible(true);
                        fieldry.set(dmYdjh,request.getAttribute("yhms"));
                    }
                }

            }
            if ("PWFK".equals(dto.getZxZtName())){
                List<YcldwInfo> ycldwInfos = ycldwInfoService.updatePwWeb(list);
                for(YcldwInfo y  :ycldwInfos){
                    wsHandler.sendPwComplete((JSONObject) JSONObject.toJSON(y));
                }

            }
            if ("HXFK".equals(dto.getZxZtName())){
                List<YcldwInfo> ycldwInfos = ycldwInfoService.updatePwWeb(list);
                for(YcldwInfo y  :ycldwInfos){
                    wsHandler.sendHXComplete((JSONObject) JSONObject.toJSON(y));
                }

            }
            if ("QGFK".equals(dto.getZxZtName())){
                //无需划线,从理料间获取时才更新垛位信息
                //if ("42".equals(dto.getZt())){
                List<YcldwInfo> ycldwInfos = ycldwInfoService.updatePwWeb(list);
                //}
                for(YcldwInfo y  :ycldwInfos){
                    wsHandler.sendHXComplete((JSONObject) JSONObject.toJSON(y));
                    wsHandler.sendQGComplete((JSONObject) JSONObject.toJSON(y));
                }

            }
            zyjhService.updateBatchById(list);



            return new ResultBean<>();
        } catch (NoSuchFieldException | IllegalAccessException e) {
            throw new ServiceException(10001,"属性字段不存在");
        }
    }

    /**
     * 派工单查询
     * 可以查询所有已经上料的数据
     */
    @PostMapping("pgd2")
    public ResultBean<?> pgd2(@RequestBody PgdSearch search) throws NoSuchFieldException {
        List<DmYdjh> list = zyjhService.list(Wrappers.<DmYdjh>query()
                        //执行查询

                .isNull(search.getField()).ge("zt",11)
//                .eq("",search.getUser()).lt("zt",62)
        );
        return new ResultBean<>(list);
    }

    /**
     * 派工单查询
     */
    @PostMapping("pgd")
    public ResultBean<?> pgd(@RequestBody PgdSearch search){
        List<DmYdjh> list = zyjhService.list(Wrappers.<DmYdjh>query()
                //执行查询
                .eq(StrUtil.isNotEmpty(search.getField()),search.getField(),search.getUser())
//                .isNull(StrUtil.isEmpty(search.getField()),search.getField())
                .eq(StrUtil.isNotEmpty(search.getZt()),"zt",search.getZt()).or(StrUtil.isNotEmpty(search.getZtHx()),it->it.eq("zt",search.getZtHx()))
                .eq(StrUtil.isNotEmpty(search.getDcch()),"dcch",search.getDcch())
                .le(StrUtil.isNotEmpty(search.getJhrqField()),search.getJhrqField(), DateUtil.date().toString("yyyy/MM/dd"))
                //跟踪查询
                .eq(StrUtil.isNotEmpty(search.getXqrq()),"xqrq",search.getXqrq())
        );
        return new ResultBean<>(list);
    }


    @PostMapping("pgdweb")
    @AuthFunction
    public ResultBean<?> pgdweb(@RequestBody PgdSearch search){
        List<DmYdjh> list = zyjhService.list(Wrappers.<DmYdjh>query()
            //执行查询
            .eq(StrUtil.isNotEmpty(search.getField()),search.getField(),search.getUser())
            .eq(StrUtil.isNotEmpty(search.getZt()),"zt",search.getZt()).or(StrUtil.isNotEmpty(search.getZtHx()),it->it.eq("zt",search.getZtHx()))
            .eq(StrUtil.isNotEmpty(search.getDcch()),"dc_ch",search.getDcch())
            .le(StrUtil.isNotEmpty(search.getJhrqField()),search.getJhrqField(), DateUtil.date().toString("yyyy/MM/dd"))
            //跟踪查询
            .eq(StrUtil.isNotEmpty(search.getXqrq()),"xqrq",search.getXqrq())
        );
        return new ResultBean<>(list);
    }


    /**
     * 获取上料计划,所有派工单数据整合到月度计划中统一管理,
     * 单独的派工单页面需要进行调整,从月度计划中获取对应工序数据
     */
    @PostMapping("sljh")
    public ResultBean<?> sljh(@RequestBody SljhSearch search){
        String date = DateUtil.date().toString("yyyy/MM/dd");
        if (StrUtil.isAllNotEmpty(search.getSljhrq())){
            date=search.getSljhrq();
        }
        List<DmYdjh> list = zyjhService.list(Wrappers.<DmYdjh>lambdaQuery()
//                .le(DmYdjh::getSljhrq, date)
                .eq(StrUtil.isNotEmpty(search.getZt()),DmYdjh::getZt, search.getZt())
                .eq(StrUtil.isNotEmpty(search.getDcch()),DmYdjh::getDcCh, search.getDcch())
                .eq(StrUtil.isNotEmpty(search.getDcPl()),DmYdjh::getDcPl, search.getDcPl())
                .eq(StrUtil.isNotEmpty(search.getKw()),DmYdjh::getKw, search.getKw())
        );
        return new ResultBean<>(list);
    }


    /**
     * 获取预处理垛位信息
     */
    @PostMapping("ycldw")
    public ResultBean<?> ycldw(String type){
        return new ResultBean<>(ycldwService.ycldwList(type));
    }

    @GetMapping("dw")
    public ResultBean<?> dw(String type){
        return new ResultBean<>(ycldwService.list(Wrappers.<Ycldw>lambdaQuery().eq(Ycldw::getType,type)));
    }


    /**
     * 上料
     */
    @PostMapping("sl")
    @Transactional(rollbackFor = Exception.class)
    public ResultBean<?> sl(@RequestBody ShangLiao sl) {
        if (!sl.checkCkwp()){
            //throw new ServiceException(10009,"物品库存不存在,或库存信息错误");
        }

        //更新预处理垛位信息
        YcldwInfo info=ycldwInfoService.updateForSl(sl);

        //更新月度计划上料相关字段
        zyjhService.updateForSl(sl);

        //todo 更新库存信息

        switch (sl.getType()){
            case YCL:
                wsHandler.sendSlComplete((JSONObject)JSONObject.toJSON(info));
                break;
            case LLJ:
                wsHandler.sendLLComplete((JSONObject)JSONObject.toJSON(info));
                break;
        }

        return new ResultBean<>();
    }

    /**
     * 倒垛
     */
    @PostMapping("daoduo")
    @Transactional(rollbackFor = Exception.class)
    public ResultBean<?> daoduo(@RequestBody DaoDuo dd){
        if (dd.getLastCeng()!=dd.getCeng()||dd.getLastDuo()!=dd.getDuo()){
            int moveY=dd.getLastCeng()-dd.getCeng();
            int y=moveY>0?1:-1;
            for (YcldwInfo info:dd.getMovedList()){
                ycldwInfoService.daoduo(info.getId(),y);
            }
            ycldwInfoService.daoduo(dd.getId(),-moveY);
        }
        return new ResultBean<>();
    }

    /**
     * 报工数据查看控制器
     * 可根据配置情况控制是否根据上道序报工后下道序才可查看到作业计划数据
     */
    @PostMapping("bgControl")
    public ResultBean<?> bgControl(String gx){
        return new ResultBean<>(zyjhService.cacheable(gx));
    }

    @PostMapping("test")
    public ResultBean<?> test(@RequestBody Message msg){
        wsHandler.tip(msg.toJson());
        if (msg.getCode()==3000){
            wsHandler.zyjhzx(msg.toJson());
        }
        return new ResultBean<>();
    }

}
