package com.dsic.gj_erp.service;

import com.dsic.gj_erp.bean.request.LoginBean;
import com.dsic.gj_erp.bean.sy.QSyfunp;
import com.dsic.gj_erp.bean.sy.QSyifrp;
import com.dsic.gj_erp.bean.sy.QSyuirp;
import com.dsic.gj_erp.bean.sy.Syuserp;
import com.dsic.gj_erp.dao.sy.SyuserpRepository;
import com.dsic.gj_erp.exception.CustomException;
import com.dsic.gj_erp.mapper.SyuserMapper;
import com.dsic.gj_erp.util.IPUtils;
import com.dsic.gj_erp.util.JwtUtil;
import com.dsic.gj_erp.util.MD5Util;
import com.querydsl.jpa.impl.JPAQueryFactory;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import javax.annotation.PostConstruct;
import javax.persistence.EntityManager;
import javax.servlet.http.HttpServletRequest;
import java.util.HashMap;
import java.util.Map;

@Slf4j
@Service
public class ApiService {

    @Autowired
    private JwtUtil jwtUtil;
    @Autowired
    SyuserpRepository syuserpRepository;

    @Autowired
    private EntityManager entityManager;

    @Autowired
    private SyuserMapper syuserMapper;

    private JPAQueryFactory queryFactory;

    @PostConstruct
    public void initFactory() {
        queryFactory = new JPAQueryFactory(entityManager);
    }


    /**
     * 用户登陆逻辑
     *
     * @param loginBean
     * @return
     * @throws CustomException
     */
    public Map<String, Object> userLogin(LoginBean loginBean, HttpServletRequest request) throws CustomException {
       String pwd="";
        String ip= IPUtils.getIpAddr(request);
        log.info("账号:{},登录IP:{}",loginBean.getYhdm(),ip);
        //String macAddress=IPUtils.getMACAddress(ip);

        if (!StringUtils.isEmpty(loginBean.getPwd())){
            pwd= MD5Util.encrypt(loginBean.getPwd());
        }

        Syuserp syuserp = syuserpRepository.findFirstByYHDMAndPWD(loginBean.getYhdm(), pwd);
        if (syuserp == null) {
            throw new CustomException(10201, "登陆失败，账号密码错误！");
        }
        if (syuserp.getIpAddress()==null ) {
            throw new CustomException(10201, "登陆失败，IP地址未配置！");
        }
        if (!syuserp.getIpAddress().equals(ip) ) {
            throw new CustomException(10201, "登陆失败，IP地址未授权！");
        }
//        if (StrUtil.isNotEmpty(syuserp.getMacAddress())&& !syuserp.getMacAddress().equals(macAddress)) {
//            throw new CustomException(10201, "登陆失败，mac地址错误！");
//        }

//        if (StrUtil.isEmpty(syuserp.getMacAddress())){
//            syuserp.setMacAddress(macAddress);
//            syuserMapper.updateUserMac(syuserp);
//        }

        syuserp.setPWD(null);

        String token = jwtUtil.generateToken(loginBean.getYhdm(), loginBean.getZxtbm(),syuserp.getYHMS());
        Map<String, Object> dataMap = new HashMap<>();
        dataMap.put("token", token);
        dataMap.put("expires", jwtUtil.getClaimByToken(token).getExpiration().getTime());
        return dataMap;
    }

    public Map<String, Object> getInfo(String yhdm) {
        Syuserp syuserp = syuserpRepository.findById(yhdm).get();
        syuserp.setPWD(null);
        Map<String, Object> dataMap = new HashMap<>();
        String[] arr = {"admin"};
        dataMap.put("roles", arr);
        dataMap.put("user", syuserp);
        return dataMap;
    }

    public Integer updatePassword(String yhdm,String newPWD) {
        return syuserMapper.updatePassword(yhdm,newPWD);
    }

    public boolean hasPermission(String yhdm, String[] ckdm) {
        if (ckdm == null || ckdm.length == 0) {
            return true;
        }
        QSyfunp syfunp = QSyfunp.syfunp;
        QSyifrp syifrp = QSyifrp.syifrp;
        QSyuirp syuirp = QSyuirp.syuirp;
        long matchCount = queryFactory.select(syifrp)
                .from(syfunp)
                .leftJoin(syifrp).on(syfunp.MS.eq(syifrp.MS).and(syfunp.ZXTBM.eq(syifrp.ZXTBM)).and(syfunp.GNLB.eq(syifrp.GNLB)))
                .leftJoin(syuirp).on(syifrp.GWDM.eq(syuirp.GWDM))
                .where(syuirp.YHDM.eq(yhdm).and(syfunp.CKDM.in(ckdm)))
                .fetchCount();
        return matchCount > 0;
    }


}
